/*
 * IRC - Internet Relay Chat, ircd/s_err.c
 * Copyright (C) 1992 Darren Reed
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sys.h"
#include "h.h"
#include "numeric.h"
#include "s_err.h"
#include "sprintf_irc.h"

RCSTAG_CC("$Id: s_err.c,v 1.5 01/07/2004 15:06:12 seaner Exp $");

typedef struct {
  int num_val;
  char *num_form;
} Numeric;

/* *INDENT-OFF* */

static Numeric local_replies[] = {
/* 000 */
    {0, (char *)NULL},
/* 001 */
 #ifdef KERNET
    {RPL_WELCOME, ":Bienvenue sur le rseau de discussion Kernet %s !"},
#else
    {RPL_WELCOME, ":Bienvenue sur le rseau de discussion %s !"},
#endif

/* 002 */
     {RPL_YOURHOST, ":Votre serveur %s, version %s"},
/* 003 */
    {RPL_CREATED, ":Ce serveur a t compil le %s"},
/* 004 */
    {RPL_MYINFO, "%s %s user-modes: dioaswkgxA ; chan-modes: biklmnopstvcCON"},
/* 005 */
    {RPL_MAP, ":%s%s"},
/* 006 */
    {RPL_MAPMORE, ":%s%s --> *plus*"},
/* 007 */
    {RPL_MAPEND, ":Fin"},
/* 008 */
    {RPL_SNOMASK, "%d :: Masque de notification du serveur (%#x)"},
/* 009 */
    {RPL_STATMEMTOT, "%u %u :Octets Blocs"},
/* 010 */
#ifdef MEMSIZESTATS
    {RPL_STATMEM, "%u %u %s %u"},
#else
    {RPL_STATMEM, "%u %u %s"},
#endif
    {0, (char *)NULL}
};

static Numeric numeric_errors[] = {
/* 401 */
    {ERR_NOSUCHNICK, "%s :Ce pseudonyme n'existe pas"},
/* 402 */
    {ERR_NOSUCHSERVER, "%s :Ce serveur n'existe pas"},
/* 403 */
    {ERR_NOSUCHCHANNEL, "%s :Ce salon n'existe pas"},
/* 404 */
    {ERR_CANNOTSENDTOCHAN, "%s :Impossible d'envoyer le message dans le salon"},
/* 405 */
    {ERR_TOOMANYCHANNELS, "%s :Vous tes dans trop de salons en mme temps"},
/* 406 */
    {ERR_WASNOSUCHNICK, "%s :Ce pseudonyme n'existait pas"},
/* 407 */
    {ERR_TOOMANYTARGETS, "%s :Plusieurs destinataires possibles. Le message n'a pas ete envoy"},
/* 408 */
    {0, (char *)NULL},
/* 409 */
    {ERR_NOORIGIN, ":Aucune origine specifie"},
/* 410 */
    {0, (char *)NULL},
/* 411 */
    {ERR_NORECIPIENT, ":Aucun destinataire (%s)"},
/* 412 */
    {ERR_NOTEXTTOSEND, ":Pas de texte a envoyer"},
/* 413 */
    {ERR_NOTOPLEVEL, "%s :Aucun domaine de premier niveau specifi"},
/* 414 */
    {ERR_WILDTOPLEVEL, "%s :Joker dans le domaine de premier niveau"},
/* 415 */
    {0, (char *)NULL},
/* 416 */
    {ERR_QUERYTOOLONG, "%s :Trop de lignes en sortie, reformulez votre demande"},
/* 417 */
           {0, (char *)NULL},
/* 418 */
    {0, (char *)NULL},
/* 419 */
    {0, (char *)NULL},
/* 420 */
    {0, (char *)NULL},
/* 421 */
    {ERR_UNKNOWNCOMMAND, "%s :Commande inconnue"},
/* 422 */
    {ERR_NOMOTD, ":"},
/* 423 */
    {ERR_NOADMININFO, "%s :Informations administratives non disponbiles"},
/* 424 */
    {0, (char *)NULL},
/* 425 */
    {0, (char *)NULL},
/* 426 */
    {0, (char *)NULL},
/* 427 */
    {0, (char *)NULL},
/* 428 */
    {0, (char *)NULL},
/* 429 */
    {0, (char *)NULL},
/* 430 */
    {0, (char *)NULL},
/* 431 */
    {ERR_NONICKNAMEGIVEN, ":Veuillez specifier un pseudonyme"},
/* 432 */
    {ERR_ERRONEUSNICKNAME, "%s :Pseudonyme erron"},
/* 433 */
    {ERR_NICKNAMEINUSE, "%s :Ce pseudonyme est deja utilis."},
/* 434 */
    {0, (char *)NULL},
/* 435 */
    {0, (char *)NULL},
/* 436 */
    {ERR_NICKCOLLISION, "%s :Collision de pseudonyme"},
/* 437 */
    {ERR_BANNICKCHANGE, "%s :Il est impossible de changer votre pseudonyme actuellement"},
/* 438 */
    {ERR_NICKTOOFAST, "%s :Changement trop rapide de votre pseudonyme. Veuillez patienter %d secondes avant de recommencer."},
/* 439 */
    {ERR_TARGETTOOFAST, "%s :Changement trop rapide de la cible. Veuillez patienter %d secondes avant de recommencer."},
/* 440 */
    {ERR_SERVICESDOWN, "%s :est actuellement dconnect. Veuillez patienter un moment et essayer aprs."},
/* 441 */
    {ERR_USERNOTINCHANNEL, "%s %s :Ils ne sont pas dans ce salon"},
/* 442 */
    {ERR_NOTONCHANNEL, "%s :Vous n'tes pas dans ce salon"},
/* 443 */
    {ERR_USERONCHANNEL, "%s %s :est dj dans le salon"},
/* 444 */
    {0, (char *)NULL},
/* 445 */
    {0, (char *)NULL},
/* 446 */
    {0, (char *)NULL},
/* 447 */
    {0, (char *)NULL},
/* 448 */
    {0, (char *)NULL},
/* 449 */
    {0, (char *)NULL},
/* 450 */
    {0, (char *)NULL},
/* 451 */
    {ERR_NOTREGISTERED, ":Vous n'tes pas referenc"},
/* 452 */
    {0, (char *)NULL},
/* 453 */
    {0, (char *)NULL},
/* 454 */
    {0, (char *)NULL},
/* 455 */
    {0, (char *)NULL},
/* 456 */
    {0, (char *)NULL},
/* 457 */
    {0, (char *)NULL},
/* 458 */
    {0, (char *)NULL},
/* 459 */
    {0, (char *)NULL},
/* 460 */
    {0, (char *)NULL},
/* 461 */
    {ERR_NEEDMOREPARAMS, "%s :Il manque des paramtres"},
/* 462 */
    {ERR_ALREADYREGISTRED, ":Vous ne devriez pas avoir  vous enregistrer  nouveau"},
/* 463 */
    {ERR_NOPERMFORHOST, ":Votre ordinateur n'a pas les privileges suffisants"},
/* 464 */
    {ERR_PASSWDMISMATCH, ":Mot de passe incorrect"},
/* 465 */
    {ERR_YOUREBANNEDCREEP, ":Vous avez t banni de ce serveur"},
/* 466 */
    {ERR_YOUWILLBEBANNED, (char *)NULL},
/* 467 */
    {ERR_KEYSET, "%s :La cle du salon existe dj"},
/* 468 */
    {ERR_INVALIDUSERNAME, (char *)NULL},
/* 469 */
    {0, (char *)NULL},
/* 470 */
    {0, (char *)NULL},
/* 471 */
    {ERR_CHANNELISFULL, "%s :Impossible de rentrer dans le salon car il est limit en nombre de personnes"},
/* 472 */
    {ERR_UNKNOWNMODE, "%c :ce mode m'est inconnu"},
/* 473 */
    {ERR_INVITEONLYCHAN, "%s :Impossible de rentrer dans le salon car vous devez tre invit pour pouvoir y accder"},
/* 474 */
    {ERR_BANNEDFROMCHAN, "%s :Impossible de rentrer dans le salon car vous tes banni"},
/* 475 */
    {ERR_BADCHANNELKEY, "%s :Impossible de rentrer dans le salon sans connaitre le mot de passe"},
/* 476 */
    {ERR_BADCHANMASK, "%s :Mauvais masque du salon"},
/* 477 */
    {0, (char *)NULL},
/* 478 */
    {ERR_BANLISTFULL, "%s %s :La liste de restrictions (ban/ignore) du salon est pleine"},
/* 479 */
    {0, (char *)NULL},
/* 480 */
    {0, (char *)NULL},
/* 481 */
    {ERR_NOPRIVILEGES, ":Permission refuse, vous n'tes pas un IRC Oprateur"},
/* 482 */
    {ERR_CHANOPRIVSNEEDED, "%s :Vous n'tes pas oprateur du salon"},
/* 483 */
    {ERR_CANTKILLSERVER, ":Vous ne pouvez pas tuer un serveur !"},
/* 484 */
    {ERR_ISCHANSERVICE, "%s %s :Il vous est impossible de doper ou de kicker un IRC Oprateur"},
/* 485 */
    {0, (char *)NULL},
/* 486 */
    {0, (char *)NULL},
/* 487 */
     {ERR_MSGSERVICES, "\002Erreur!\002 \"/msg %s\" n'est pas autoris. Veuillez utiliser \"/msg %s@%s\" ou \"/%s\"."}, // nick, nick, service_names, bot_cmd
/* 488 */
    {0, (char *)NULL},
/* 489 */
    {ERR_VOICENEEDED, "%s :Vous n'avez pas les privilges suffisants"},
/* 490 */
    {0, (char *)NULL},
/* 491 */
    {ERR_NOOPERHOST, ":Aucune O-Line pour votre host !"},
/* 492 */
    {0, (char *)NULL},
/* 493 */
    {0, (char *)NULL},
/* 494 */
    {0, (char *)NULL},
/* 495 */
    {0, (char *)NULL},
/* 496 */
    {0, (char *)NULL},
/* 497 */
    {0, (char *)NULL},
/* 498 */
    {0, (char *)NULL},
/* 499 */
    {0, (char *)NULL},
/* 500 */
    {0, (char *)NULL},
/* 501 */
    {ERR_UMODEUNKNOWNFLAG, ":Attribut inconnu"},
/* 502 */
    {ERR_USERSDONTMATCH, ":Il n'est pas permis de changer les attributs d'une autre personne"},
/* 503 */
    {0, (char *)NULL},
/* 504 */
    {0, (char *)NULL},
/* 505 */
    {0, (char *)NULL},
/* 506 */
    {0, (char *)NULL},
/* 507 */
    {0, (char *)NULL},
/* 508 */
    {0, (char *)NULL},
/* 509 */
    {0, (char *)NULL},
/* 510 */
    {ERR_OPERONLY, "%s :Impossible de joindre le salon, vous devez tre IRC Oprateur (+O)"},
/* 511 */
    {ERR_SILELISTFULL, "%s :Votre liste de muets est pleine"},
/* 512 */
    {ERR_NOSUCHGLINE, "%s@%s :Ne correspond a aucun G-Line"},
/* 513 */
    {ERR_BADPING, (char *)NULL}
};

static Numeric numeric_replies[] = {
/* 300 */
    {RPL_NONE, (char *)NULL},
/* 301 */
    {RPL_AWAY, "%s :%s"},
/* 302 */
    {RPL_USERHOST, ":"},
/* 303 */
    {RPL_ISON, ":"},
/* 304 */
    {RPL_TEXT, (char *)NULL},
/* 305 */
    {RPL_UNAWAY, ":Vous tes de nouveau parmi nous"},
/* 306 */
    {RPL_NOWAWAY, ":Vous vous absentez"},
/* 307 */
    {RPL_USERIP, ":"},
/* 308 */ /* Inusit, voir whocmds.c - StyX */
    {RPL_WHOISADMIN, "%s :est un IRC Oprateur"},
/* 309 */
    {0, (char *)NULL},
/* 310 */
    {0, (char *)NULL},
/* 311 */
    {RPL_WHOISUSER, "%s %s %s * :%s"},
/* 312 */
    {RPL_WHOISSERVER, "%s %s :%s"},
/* 313 */
    {RPL_WHOISOPERATOR, "%s :is an IRC Operator%s"},
/* 314 */
    {RPL_WHOWASUSER, "%s %s %s * :%s"},
/* 315 */
    {RPL_ENDOFWHO, "%s :Fin de la liste."},
/* 316 */
    {0, (char *)NULL},
/* 317 */
    {RPL_WHOISIDLE, "%s %ld %ld :seconds idle, signon time"},
/* 318 */
    {RPL_ENDOFWHOIS, "%s :Fin de la liste."},
/* 319 */
    {RPL_WHOISCHANNELS, "%s :%s"},
/* 320 */
    {0, (char *)NULL},
/* 321 */
    {RPL_LISTSTART, "Salon :Utilisateurs  Nom"},
/* 322 */
    {RPL_LIST, "%s %d :%s"},
/* 323 */
    {RPL_LISTEND, ":Fin de la liste."},
/* 324 */
    {RPL_CHANNELMODEIS, "%s %s %s"},
/* 325 */
    {0, (char *)NULL},
/* 326 */
    {0, (char *)NULL},
/* 327 */
    {0, (char *)NULL},
/* 328 */
    {0, (char *)NULL},
/* 329 */
    {RPL_CREATIONTIME, "%s %lu"},
/* 330 */
    {0, (char *)NULL},
/* 331 */
    {RPL_NOTOPIC, "%s :Le sujet n'a pas t chang."},
/* 332 */
    {RPL_TOPIC, "%s :%s"},
/* 333 */
    {RPL_TOPICWHOTIME, "%s %s %lu"},
/* 334 */
    {RPL_LISTUSAGE, ":%s"},
/* 335 */
    {0, (char *)NULL},
/* 336 */
    {0, (char *)NULL},
/* 337 */
    {0, (char *)NULL},
/* 338 */
    {0, (char *)NULL},
/* 339 */
    {0, (char *)NULL},
/* 340 */
    {0, (char *)NULL},
/* 341 */
    {RPL_INVITING, "%s %s"},
/* 342 */
    {0, (char *)NULL},
/* 343 */
    {0, (char *)NULL},
/* 344 */
    {0, (char *)NULL},
/* 345 */
    {0, (char *)NULL},
/* 346 */
    {0, (char *)NULL},
/* 347 */
    {0, (char *)NULL},
/* 348 */
    {0, (char *)NULL},
/* 349 */
    {0, (char *)NULL},
/* 350 */
    {0, (char *)NULL},
/* 351 */
    {RPL_VERSION, "%s.%s %s :%s"},
/* 352 */
    {RPL_WHOREPLY, "%s"},
/* 353 */
    {RPL_NAMREPLY, "%s"},
/* 354 */
    {RPL_WHOSPCRPL, "%s"},
/* 355 */
    {0, (char *)NULL},
/* 356 */
    {0, (char *)NULL},
/* 357 */
    {0, (char *)NULL},
/* 358 */
    {0, (char *)NULL},
/* 359 */
    {0, (char *)NULL},
/* 360 */
    {0, (char *)NULL},
/* 361 */
    {RPL_KILLDONE, (char *)NULL},
/* 362 */
    {RPL_CLOSING, "%s :Ferme. Statut = %d"},
/* 363 */
    {RPL_CLOSEEND, "%d: Connexions fermes"},
/* 364 */
#ifndef GODMODE
    {RPL_LINKS, "%s %s :%d P%u %s"},
#else /* GODMODE */
    {RPL_LINKS, "%s %s :%d P%u %lu (%c) %s"},
#endif /* GODMODE */
/* 365 */
    {RPL_ENDOFLINKS, "%s :Fin de la liste."},
/* 366 */
    {RPL_ENDOFNAMES, "%s :Fin de la liste."},
/* 367 */
    {RPL_BANLIST, "%s %s %s %lu"},
/* 368 */
    {RPL_ENDOFBANLIST, "%s :Fin de la liste"},
/* 369 */
    {RPL_ENDOFWHOWAS, "%s :Fin de la liste"},
/* 370 */
    {0, (char *)NULL},
/* 371 */
    {RPL_INFO, ":%s"},
/* 372 */
    {RPL_MOTD, ":- %s"},
/* 373 */
    {RPL_INFOSTART, ":Informations Serveur"},
/* 374 */
    {RPL_ENDOFINFO, ":Fin des informations serveur."},
/* 375 */
    {RPL_MOTDSTART, ":- %s Message du jour - "},
/* 376 */
    {RPL_ENDOFMOTD, ":Fin de la commande."},
/* 377 */
    {0, (char *)NULL},
/* 378 */
    {RPL_OMOTDSTART, ":- Message du jour des IRC Oprateurs - "},
/* 379 */
    {RPL_OMOTD, ":- %s"},
/* 380 */
    {RPL_ENDOFOMOTD, ":Fin du message du jour des IRC Oprateurs"},
/* 381 */
    {RPL_YOUREOPER, ":Vous tes maintenant un IRC Oprateur."},
/* 382 */
    {RPL_REHASHING, "%s :Rechargement"},
/* 383 */
    {0, (char *)NULL},
/* 384 */
    {RPL_MYPORTIS, "%d :Le port local du serveur est\r\n"},
/* 385 */
    {RPL_NOTOPERANYMORE, (char *)NULL},
/* 386 */
    {0, (char *)NULL},
/* 387 */
    {0, (char *)NULL},
/* 388 */
    {0, (char *)NULL},
/* 389 */
    {0, (char *)NULL},
/* 390 */
    {0, (char *)NULL},
/* 391 */
    {RPL_TIME, "%s %lu %ld :%s"},
/* 392 */
    {0, (char *)NULL},
/* 393 */
    {0, (char *)NULL},
/* 394 */
    {0, (char *)NULL},
/* 395 */
    {0, (char *)NULL},
/* 396 */
    {0, (char *)NULL},
/* 397 */
    {0, (char *)NULL},
/* 398 */
    {0, (char *)NULL},
/* 399 */
    {0, (char *)NULL},
/* 200 */
#ifndef GODMODE
    {RPL_TRACELINK, "Link %s%s %s %s"},
#else /* GODMODE */
    {RPL_TRACELINK, "Link %s%s %s %s %lu"},
#endif /* GODMODE */
/* 201 */
    {RPL_TRACECONNECTING, "Essai. %d %s"},
/* 202 */
    {RPL_TRACEHANDSHAKE, "H.S. %d %s"},
/* 203 */
    {RPL_TRACEUNKNOWN, "???? %d %s"},
/* 204 */
    {RPL_TRACEOPERATOR, "Oper %d %s %ld"},
/* 205 */
    {RPL_TRACEUSER, "User %d %s %ld"},
/* 206 */
    {RPL_TRACESERVER, "Serv %d %dS %dC %s %s!%s@%s %ld %ld"},
/* 207 */
    {0, (char *)NULL},
/* 208 */
    {RPL_TRACENEWTYPE, "<newtype> 0 %s"},
/* 209 */
    {RPL_TRACECLASS, "Class %d %d"},
/* 210 */
    {0, (char *)NULL},
/* 211 */
    {RPL_STATSLINKINFO, (char *)NULL},
/* 212 */
    {RPL_STATSCOMMANDS, "%s %u %u"},
/* 213 */
    {RPL_STATSCLINE, "%c %s * %s %d %d"},
/* 214 */
    {RPL_STATSNLINE, "%c %s * %s %d %d"},
/* 215 */
    {RPL_STATSILINE, "%c %s * %s %d %d"},
/* 216 */
    {RPL_STATSKLINE, "%c %s %s %s %d %d"},
/* 217 */
    {RPL_STATSPLINE, "%c %d %d %#x"},
/* 218 */
    {RPL_STATSYLINE, "%c %d %d %d %d %ld"},
/* 219 */
    {RPL_ENDOFSTATS, "%c :Fin des statistiques."},
/* 220 */
    {0, (char *)NULL},
/* 221 */
    {RPL_UMODEIS, "%s"},
/* 222 */
    {0, (char *)NULL},
/* 223 */
    {0, (char *)NULL},
/* 224 */
    {0, (char *)NULL},
/* 225 */
    {0, (char *)NULL},
/* 226 */
    {0, (char *)NULL},
/* 227 */
    {0, (char *)NULL},
/* 228 */
    {0, (char *)NULL},
/* 229 */
    {0, (char *)NULL},
/* 230 */
    {0, (char *)NULL},
/* 231 */
    {RPL_SERVICEINFO, (char *)NULL},
/* 232 */
    {RPL_ENDOFSERVICES, (char *)NULL},
/* 233 */
    {RPL_SERVICE, (char *)NULL},
/* 234 */
    {RPL_SERVLIST, (char *)NULL},
/* 235 */
    {RPL_SERVLISTEND, (char *)NULL},
/* 236 */
    {0, (char *)NULL},
/* 237 */
    {0, (char *)NULL},
/* 238 */
    {0, (char *)NULL},
/* 239 */
    {0, (char *)NULL},
/* 240 */
    {0, (char *)NULL},
/* 241 */
    {RPL_STATSLLINE, "%c %s * %s %d %d"},
/* 242 */
    {RPL_STATSUPTIME, ":Server Up %d days, %d:%02d:%02d"},
/* 243 */
    {RPL_STATSOLINE, "%c %s * %s %d %d"},
/* 244 */
    {RPL_STATSHLINE, "%c %s * %s %d %d"},
/* 245 */
    {0, (char *)NULL},
/* 246 */
    {RPL_STATSTLINE, "%c %s %s"},
/* 247 */
    {RPL_STATSGLINE, "%c %s@%s %lu :%s"},
/* 248 */
    {RPL_STATSULINE, "%c %s %s %s %d %d"},
/* 249 */
    {0, (char *)NULL},
/* 250 */
    {RPL_STATSCONN, ":Maximum de connexions reues: %d (%d chatteurs)"},
/* 251 */
    {RPL_LUSERCLIENT, ":Il y a %d chatteurs et %d chatteurs invisibles sur %d serveur(s)"},
/* 252 */
    {RPL_LUSEROP, "%d :oprateur(s) en ligne"},
/* 253 */
    {RPL_LUSERUNKNOWN, "%d :connexion(s) inconnu(s)"},
/* 254 */
    {RPL_LUSERCHANNELS, "%d :salons ouvert"},
/* 255 */
    {RPL_LUSERME, ":J'ai %d utilisateurs et %d serveurs"},
/* 256 */
    {RPL_ADMINME, ":Informations administratives sur %s"},
/* 257 */
    {RPL_ADMINLOC1, ":%s"},
/* 258 */
    {RPL_ADMINLOC2, ":%s"},
/* 259 */
    {RPL_ADMINEMAIL, ":%s"},
/* 260 */
    {0, (char *)NULL},
/* 261 */
    {RPL_TRACELOG, "Fichier %s %d"},
/* 262 */
    {RPL_TRACEPING, "Ping %s %s"},
/* 263 */
    {0, (char *)NULL},
/* 264 */
    {0, (char *)NULL},
/* 265 */
    {0, (char *)NULL},
/* 266 */
    {0, (char *)NULL},
/* 267 */
    {0, (char *)NULL},
/* 268 */
    {0, (char *)NULL},
/* 269 */
    {0, (char *)NULL},
/* 270 */
    {0, (char *)NULL},
/* 271 */
    {RPL_SILELIST, "%s %s"},
/* 272 */
    {RPL_ENDOFSILELIST, "%s :Fin de la liste"},
/* 273 */
    {0, (char *)NULL},
/* 274 */
    {0, (char *)NULL},
/* 275 */
    {RPL_STATSDLINE, "%c %s %s"},
/* 276 */
    {0, (char *)NULL},
/* 277 */
    {0, (char *)NULL},
/* 278 */
    {0, (char *)NULL},
/* 279 */
    {0, (char *)NULL},
/* 280 */
    {RPL_GLIST, "%s@%s %lu %s%s"},
/* 281 */
    {RPL_ENDOFGLIST, ":Fin de la liste"}
};

/* *INDENT-ON* */

static char numbuff[512];

/* "inline" */
#define prepbuf(buffer, num, tail)			\
{							\
  register char *s = buffer + 4;			\
  register const char *ap = atoi_tab + (num << 2);	\
  							\
  strcpy(buffer, ":%s 000 %s ");			\
  *s++ = *ap++;						\
  *s++ = *ap++;						\
  *s = *ap;						\
  strcpy(s + 5, tail);					\
}

char *err_str(int numeric)
{
  Reg1 Numeric *nptr;
  Reg2 int num = numeric;

  num -= numeric_errors[0].num_val;

#ifdef DEBUGMODE
  if (num < 0 || num > ERR_USERSDONTMATCH)
    sprintf_irc(numbuff,
	":%%s %d %%s :INTERNAL ERROR: BAD NUMERIC! %d", numeric, num);
  else
  {
    nptr = &numeric_errors[num];
    if (!nptr->num_form || !nptr->num_val)
      sprintf_irc(numbuff, ":%%s %d %%s :NO ERROR FOR NUMERIC ERROR %d",
	  numeric, num);
    else
      prepbuf(numbuff, nptr->num_val, nptr->num_form);
  }
#else
  nptr = &numeric_errors[num];
  prepbuf(numbuff, nptr->num_val, nptr->num_form);
#endif

  return numbuff;
}

char *rpl_str(int numeric)
{
  Reg1 Numeric *nptr;
  Reg2 int num = numeric;

  if (num > (int)(sizeof(local_replies) / sizeof(Numeric) - 2))
    num -= (num > 300) ? 300 : 100;

#ifdef DEBUGMODE
  if (num < 0 || num > 200)
    sprintf_irc(numbuff, ":%%s %d %%s :INTERNAL REPLY ERROR: BAD NUMERIC! %d",
	numeric, num);
  else
  {
    if (numeric > 99)
      nptr = &numeric_replies[num];
    else
      nptr = &local_replies[num];
    Debug((DEBUG_NUM, "rpl_str: numeric %d num %d nptr %p %d %p",
	numeric, num, nptr, nptr->num_val, nptr->num_form));
    if (!nptr->num_form || !nptr->num_val)
      sprintf_irc(numbuff, ":%%s %d %%s :NO REPLY FOR NUMERIC ERROR %d",
	  numeric, num);
    else
      prepbuf(numbuff, nptr->num_val, nptr->num_form);
  }
#else
  if (numeric > 99)
    nptr = &numeric_replies[num];
  else
    nptr = &local_replies[num];
  prepbuf(numbuff, nptr->num_val, nptr->num_form);
#endif

  return numbuff;
}


