/*
 * IRC - Internet Relay Chat, ircd/s_misc.c (formerly ircd/date.c)
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sys.h"
#include <sys/stat.h>
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef USE_SYSLOG
#include <syslog.h>
#endif
#include "h.h"
#include "struct.h"
#include "s_serv.h"
#include "numeric.h"
#include "send.h"
#include "s_conf.h"
#include "s_misc.h"
#include "common.h"
#include "match.h"
#include "hash.h"
#include "s_bsd.h"
#include "res.h"
#include "list.h"
#include "ircd.h"
#include "s_ping.h"
#include "channel.h"
#include "s_err.h"
#include "support.h"
#include "userload.h"
#include "parse.h"
#include "s_user.h"
#include "numnicks.h"
#include "sprintf_irc.h"
#include "querycmds.h"
#include "IPcheck.h"
/* Seaner hp */

/* PirO hp  
* bas sur une alghorthime en SHA 
* (1028 bits * 128 bits) source code issu
* du daemon : "shacrypt"
* (dispo sur sourceforge.net) 
* principe :
* HostNonCrypt -> hachage -> conversion en pile mmoire
* -> cryptage -> conversion inverse -> HostCrypt
* la cl de hashage est ffectue en fontion de l'host de manire 
* a obtenir toujour la meme cl pour la meme host , donc le meme cryptage.
*/



#define SIZEOF_INT 4

/* The number of bytes in a long.  */
#define SIZEOF_LONG 4

/* The number of bytes in a short.  */
#define SIZEOF_SHORT 2

/* The number of bytes in a size_t.  */
#define SIZEOF_SIZE_T 4

#if SIZEOF_INT == 4
  typedef unsigned int uint32t;
#else
# if SIZEOF_LONG == 4
   typedef unsigned long uint32t;
# else
#  if SIZEOF_SHORT == 4
    typedef unsigned short uint32t;
#  else
#   error "unable to find 32-bit data type"
#  endif /* SHORT != 4 */
# endif /* LONG != 4 */
#endif /* INT != 4 */

#define SHA_DATASIZE    64
#define SHA_DATALEN     16
#define SHA_DIGESTSIZE  20
#define SHA_DIGESTLEN    10

#define f1(x,y,z)   ( z ^ ( x & ( y ^ z ) ) )           /* Rounds  0-19 */
#define f2(x,y,z)   ( x ^ y ^ z )                       /* Rounds 20-39 */
#define f3(x,y,z)   ( ( x & y ) | ( z & ( x | y ) ) )   /* Rounds 40-59 */
#define f4(x,y,z)   ( x ^ y ^ z )                       /* Rounds 60-79 */

#define K1  0x5A827999L                                 /* Rounds  0-19 */
#define K2  0x6ED9EBA1L                                 /* Rounds 20-39 */
#define K3  0x8F1BBCDCL                                 /* Rounds 40-59 */
#define K4  0xCA62C1D6L                                 /* Rounds 60-79 */

#define h0init  0x67452301L
#define h1init  0xEFCDAB89L
#define h2init  0x98BADCFEL
#define h3init  0x10325476L
#define h4init  0xC3D2E1F0L

#define ROTL(n,X)  ( ( (X) << (n) ) | ( (X) >> ( 32 - (n) ) ) )

#define expand(W,i) ( W[ i & 15 ] = \
		      ROTL( 1, ( W[ i & 15 ] ^ W[ (i - 14) & 15 ] ^ \
				 W[ (i - 8) & 15 ] ^ W[ (i - 3) & 15 ] ) ) )

#define subRound(a, b, c, d, e, f, k, data) \
    ( e += ROTL( 5, a ) + f( b, c, d ) + k + data, b = ROTL( 30, b ) )

struct sha_ctx {
  uint32t digest[SHA_DIGESTLEN];  /* Message digest */
  uint32t count_l, count_h;       /* 64-bit block count */
  int index;                             /* index into buffer */
  char block[SHA_DATASIZE];     /* SHA data buffer */
};

static void sha_transform(struct sha_ctx *ctx, uint32t *data)
{
  uint32t A, B, C, D, E;     /* Local vars */

  /* Set up first buffer and local data buffer */
  A = ctx->digest[0];
  B = ctx->digest[1];
  C = ctx->digest[2];
  D = ctx->digest[3];
  E = ctx->digest[4];

  /* Heavy mangling, in 4 sub-rounds of 20 interations each. */
  subRound( A, B, C, D, E, f1, K1, data[ 0] );
  subRound( E, A, B, C, D, f1, K1, data[ 1] );
  subRound( D, E, A, B, C, f1, K1, data[ 2] );
  subRound( C, D, E, A, B, f1, K1, data[ 3] );
  subRound( B, C, D, E, A, f1, K1, data[ 4] );
  subRound( A, B, C, D, E, f1, K1, data[ 5] );
  subRound( E, A, B, C, D, f1, K1, data[ 6] );
  subRound( D, E, A, B, C, f1, K1, data[ 7] );
  subRound( C, D, E, A, B, f1, K1, data[ 8] );
  subRound( B, C, D, E, A, f1, K1, data[ 9] );
  subRound( A, B, C, D, E, f1, K1, data[10] );
  subRound( E, A, B, C, D, f1, K1, data[11] );
  subRound( D, E, A, B, C, f1, K1, data[12] );
  subRound( C, D, E, A, B, f1, K1, data[13] );
  subRound( B, C, D, E, A, f1, K1, data[14] );
  subRound( A, B, C, D, E, f1, K1, data[15] );
  subRound( E, A, B, C, D, f1, K1, expand( data, 16 ) );
  subRound( D, E, A, B, C, f1, K1, expand( data, 17 ) );
  subRound( C, D, E, A, B, f1, K1, expand( data, 18 ) );
  subRound( B, C, D, E, A, f1, K1, expand( data, 19 ) );

  subRound( A, B, C, D, E, f2, K2, expand( data, 20 ) );
  subRound( E, A, B, C, D, f2, K2, expand( data, 21 ) );
  subRound( D, E, A, B, C, f2, K2, expand( data, 22 ) );
  subRound( C, D, E, A, B, f2, K2, expand( data, 23 ) );
  subRound( B, C, D, E, A, f2, K2, expand( data, 24 ) );
  subRound( A, B, C, D, E, f2, K2, expand( data, 25 ) );
  subRound( E, A, B, C, D, f2, K2, expand( data, 26 ) );
  subRound( D, E, A, B, C, f2, K2, expand( data, 27 ) );
  subRound( C, D, E, A, B, f2, K2, expand( data, 28 ) );
  subRound( B, C, D, E, A, f2, K2, expand( data, 29 ) );
  subRound( A, B, C, D, E, f2, K2, expand( data, 30 ) );
  subRound( E, A, B, C, D, f2, K2, expand( data, 31 ) );
  subRound( D, E, A, B, C, f2, K2, expand( data, 32 ) );
  subRound( C, D, E, A, B, f2, K2, expand( data, 33 ) );
  subRound( B, C, D, E, A, f2, K2, expand( data, 34 ) );
  subRound( A, B, C, D, E, f2, K2, expand( data, 35 ) );
  subRound( E, A, B, C, D, f2, K2, expand( data, 36 ) );
  subRound( D, E, A, B, C, f2, K2, expand( data, 37 ) );
  subRound( C, D, E, A, B, f2, K2, expand( data, 38 ) );
  subRound( B, C, D, E, A, f2, K2, expand( data, 39 ) );

  subRound( A, B, C, D, E, f3, K3, expand( data, 40 ) );
  subRound( E, A, B, C, D, f3, K3, expand( data, 41 ) );
  subRound( D, E, A, B, C, f3, K3, expand( data, 42 ) );
  subRound( C, D, E, A, B, f3, K3, expand( data, 43 ) );
  subRound( B, C, D, E, A, f3, K3, expand( data, 44 ) );
  subRound( A, B, C, D, E, f3, K3, expand( data, 45 ) );
  subRound( E, A, B, C, D, f3, K3, expand( data, 46 ) );
  subRound( D, E, A, B, C, f3, K3, expand( data, 47 ) );
  subRound( C, D, E, A, B, f3, K3, expand( data, 48 ) );
  subRound( B, C, D, E, A, f3, K3, expand( data, 49 ) );
  subRound( A, B, C, D, E, f3, K3, expand( data, 50 ) );
  subRound( E, A, B, C, D, f3, K3, expand( data, 51 ) );
  subRound( D, E, A, B, C, f3, K3, expand( data, 52 ) );
  subRound( C, D, E, A, B, f3, K3, expand( data, 53 ) );
  subRound( B, C, D, E, A, f3, K3, expand( data, 54 ) );
  subRound( A, B, C, D, E, f3, K3, expand( data, 55 ) );
  subRound( E, A, B, C, D, f3, K3, expand( data, 56 ) );
  subRound( D, E, A, B, C, f3, K3, expand( data, 57 ) );
  subRound( C, D, E, A, B, f3, K3, expand( data, 58 ) );
  subRound( B, C, D, E, A, f3, K3, expand( data, 59 ) );

  subRound( A, B, C, D, E, f4, K4, expand( data, 60 ) );
  subRound( E, A, B, C, D, f4, K4, expand( data, 61 ) );
  subRound( D, E, A, B, C, f4, K4, expand( data, 62 ) );
  subRound( C, D, E, A, B, f4, K4, expand( data, 63 ) );
  subRound( B, C, D, E, A, f4, K4, expand( data, 64 ) );
  subRound( A, B, C, D, E, f4, K4, expand( data, 65 ) );
  subRound( E, A, B, C, D, f4, K4, expand( data, 66 ) );
  subRound( D, E, A, B, C, f4, K4, expand( data, 67 ) );
  subRound( C, D, E, A, B, f4, K4, expand( data, 68 ) );
  subRound( B, C, D, E, A, f4, K4, expand( data, 69 ) );
  subRound( A, B, C, D, E, f4, K4, expand( data, 70 ) );
  subRound( E, A, B, C, D, f4, K4, expand( data, 71 ) );
  subRound( D, E, A, B, C, f4, K4, expand( data, 72 ) );
  subRound( C, D, E, A, B, f4, K4, expand( data, 73 ) );
  subRound( B, C, D, E, A, f4, K4, expand( data, 74 ) );
  subRound( A, B, C, D, E, f4, K4, expand( data, 75 ) );
  subRound( E, A, B, C, D, f4, K4, expand( data, 76 ) );
  subRound( D, E, A, B, C, f4, K4, expand( data, 77 ) );
  subRound( C, D, E, A, B, f4, K4, expand( data, 78 ) );
  subRound( B, C, D, E, A, f4, K4, expand( data, 79 ) );

  /* Build message digest */
  ctx->digest[0] += A;
  ctx->digest[1] += B;
  ctx->digest[2] += C;
  ctx->digest[3] += D;
  ctx->digest[4] += E;
}

#define EXTRACT_UCHAR(p)  (*(unsigned char *)(p))
#define STRING2INT(s) ((((((EXTRACT_UCHAR(s) << 8)    \
			 | EXTRACT_UCHAR(s+1)) << 8)  \
			 | EXTRACT_UCHAR(s+2)) << 8)  \
			 | EXTRACT_UCHAR(s+3))

static void sha_block(struct sha_ctx *ctx, char *block)
{
  uint32t data[SHA_DATALEN];
  int i;
  
  /* Update block count */
  if (!++ctx->count_l)
    ++ctx->count_h;

  /* Endian independent conversion */
  for (i = 0; i<SHA_DATALEN; i++, block += 4)
    data[i] = STRING2INT(block);

  sha_transform(ctx, data);
}

static void make_sha(char *buffer, uint32t len, uint32t *s)
{
  struct sha_ctx ctxbuf = {
    {h0init, h1init, h2init, h3init, h4init},
    0, 0, 0,
  };
  int i, words;
  uint32t data[SHA_DATALEN];
  
  while (len >= SHA_DATASIZE)
    {
      sha_block(&ctxbuf, buffer);
      buffer += SHA_DATASIZE;
      len -= SHA_DATASIZE;
    }
  if ((ctxbuf.index = len))     /* This assignment is intended */
    /* Buffer leftovers */
    memmove(ctxbuf.block, buffer, len);
  
  /*** sha_final ***/
  
  i = ctxbuf.index;
  /* Set the first char of padding to 0x80.  This is safe since there is
     always at least one byte free */
  ctxbuf.block[i++] = 0x80;

  /* Fill rest of word */
  for( ; i & 3; i++)
    ctxbuf.block[i] = 0;

  /* i is now a multiple of the word size 4 */
  words = i >> 2;
  for (i = 0; i < words; i++)
    data[i] = STRING2INT(ctxbuf.block + 4*i);
  
  if (words > (SHA_DATALEN-2))
    { /* No room for length in this block. Process it and
       * pad with another one */
      for (i = words ; i < SHA_DATALEN; i++)
	data[i] = 0;
      sha_transform(&ctxbuf, data);
      for (i = 0; i < (SHA_DATALEN-2); i++)
	data[i] = 0;
    }
  else
    for (i = words ; i < SHA_DATALEN - 2; i++)
      data[i] = 0;
  /* Theres 512 = 2^9 bits in one block */
  data[SHA_DATALEN-2] = (ctxbuf.count_h << 9) | (ctxbuf.count_l >> 23);
  data[SHA_DATALEN-1] = (ctxbuf.count_l << 9) | (ctxbuf.index << 3);
  sha_transform(&ctxbuf, data);
  
  /*
   * we don't want to use the digest as 20x8-bit character array, but
   * as 5x32-bit integer array, so we use this to make it endian-safe
   * for *our* purposes
   */
  memcpy(s, ctxbuf.digest, SHA_DIGESTSIZE);
}

#define KEYSIZE 128 

unsigned char hostprotkey[KEYSIZE];
/* generates random data and fills the key buffer */
void make_hostprotkey()
{
  int fd;
  int i;
  fd = open("/dev/urandom", O_RDONLY);
  if (fd >= 0)
  {  /* we have a good /dev/urandom, read from it */
    if (read(fd, hostprotkey, KEYSIZE) == KEYSIZE)
    {
      close(fd);
      return;
    }
    /* there was an error reading, fall back to random() */
    close(fd);
  }
  /* i hope the following satisfies different implementations of random() */
  for (i = 0; i < KEYSIZE; i++)
    hostprotkey[i] = (unsigned char) ((random() & 0xff0000) >> 16);
}

static void make_digest_len(const char *src, uint32t *digest, int len)
{
  char buf[KEYSIZE + HOSTLEN];
  int i, k;

  i = KEYSIZE / 2;
  k = KEYSIZE - i;
  memcpy(buf, hostprotkey, i);
  memcpy(buf + i, src, len);
  memcpy(buf + i + len, hostprotkey + i, k);
  
  make_sha(buf, i + len + k, digest);
}

static void make_digest(const char *src, uint32t *digest)
{
  make_digest_len(src, digest, strlen(src));
}

static uint32t make_sum_from_digest(uint32t *digest)
{
  uint32t sum;
  
  sum = digest[0] ^ digest[1] ^ digest[2] ^ digest[3] ^ digest[4];
  /* instead of a simple (sum %= 0x7ffff), we use.. */
  sum = (sum & 0xFFFFFFFF) ^ ((sum & 0x4FFFFFFF) >> 13);
 /* sum = (sum & 0xFFFFFFFFFFF) ^ ((sum & 0x4FFFFFFF) >> 13); */
  /* this will give a number in the range 0 - 524287 */
  return sum;
}

char *makeHash(const char *mask)
{
static char final[HOSTLEN+1];

  uint32t digest[SHA_DIGESTLEN];
  uint32t sum;
  
  make_digest(mask, digest);
  sum = make_sum_from_digest(digest);
  
  sprintf(final, "%lu", (unsigned long) sum); 
  return final;
}

char *hostprotcrypt(const char *mask)
{
  char host[HOSTLEN+1];
  static char final[HOSTLEN+1];
  char * p, *crypted;
  
  strcpy(host, mask);
	/* stage 1 : mask IP -> int */
  if(check_if_ipmask(host)) {
		char *prec = host;
		int num = 0;
		while((p = strchr(prec,'.')) != NULL) {
			*p = '\0';
			num <<= 8;
			num |= (atoi(prec) & 0xFF); 
			prec = p + 1;
		}			
		num <<= 8;
		num |= (atoi(prec) & 0xFF); 
		
	
		/* stage 2 : IP int -> string */
		{
			char toCrypt[9];
			int i;
			for(i = 0; i < 5; i++) {
				toCrypt[i] = (num & 0xF) + 32;
				num >>= 4;
			}
			toCrypt[8] = '\0';
			

		/* stage 3 string -> crypt */
			{
			char *crypted = makeHash(toCrypt);
		
		/* stage 4 crypt -> look like an adresse*/
			{
				int j = 0;
				for(i = 0;i < 9;i++) {	
					final[j++] = crypted[i];
					if(i == 2 || i == 4 || i ==6 ) {
						final[j++] = '.';
					}
				}
				final[j] = '\0';
			}
		}
	}		
  } else {
  	p=strrchr(host,'.');
  	
  	*p = '\0';
	crypted = (char *)makeHash(host); 
		strcpy(final, crypted); 
		strcat(final,".");
		strcat(final,p + 1);		
  }
	return final;
}
 /* Bejita */
char *hostprot(anUser *user) {
if(user->crypt[0] == '\0') {
      strcpy(user->crypt,hostprotcrypt(user->host));
      }
        return user->crypt;
   }

/* Seaner hp */


RCSTAG_CC("$Id: s_misc.c,v 1.8 2000/06/13 15:15:35 echo Exp $");

static void exit_one_client(aClient *, char *);

static char *months[] = {
  "January", "February", "March", "April",
  "May", "June", "July", "August",
  "September", "October", "November", "December"
};

static char *weekdays[] = {
  "Sunday", "Monday", "Tuesday", "Wednesday",
  "Thursday", "Friday", "Saturday"
};

/*
 * stats stuff
 */
struct stats ircst, *ircstp = &ircst;

char *date(time_t clock)
{
  static char buf[80], plus;
  Reg1 struct tm *lt, *gm;
  struct tm gmbuf;
  int minswest;

  if (!clock)
    clock = now;
  gm = gmtime(&clock);
  memcpy(&gmbuf, gm, sizeof(gmbuf));
  gm = &gmbuf;
  lt = localtime(&clock);

  if (lt->tm_yday == gm->tm_yday)
    minswest = (gm->tm_hour - lt->tm_hour) * 60 +
	(gm->tm_min - lt->tm_min);
  else if (lt->tm_yday > gm->tm_yday)
    minswest = (gm->tm_hour - (lt->tm_hour + 24)) * 60;
  else
    minswest = ((gm->tm_hour + 24) - lt->tm_hour) * 60;

  plus = (minswest > 0) ? '-' : '+';
  if (minswest < 0)
    minswest = -minswest;

  sprintf(buf, "%s %s %d %d -- %02d:%02d %c%02d:%02d",
      weekdays[lt->tm_wday], months[lt->tm_mon], lt->tm_mday,
      1900 + lt->tm_year, lt->tm_hour, lt->tm_min,
      plus, minswest / 60, minswest % 60);

  return buf;
}

/*
 * myctime
 *
 * This is like standard ctime()-function, but it zaps away
 * the newline from the end of that string. Also, it takes
 * the time value as parameter, instead of pointer to it.
 * Note that it is necessary to copy the string to alternate
 * buffer (who knows how ctime() implements it, maybe it statically
 * has newline there and never 'refreshes' it -- zapping that
 * might break things in other places...)
 */
char *myctime(time_t value)
{
  static char buf[28];
  Reg1 char *p;

  strcpy(buf, ctime(&value));
  if ((p = strchr(buf, '\n')) != NULL)
    *p = '\0';

  return buf;
}

/*
 *  get_client_name
 *       Return the name of the client for various tracking and
 *       admin purposes. The main purpose of this function is to
 *       return the "socket host" name of the client, if that
 *    differs from the advertised name (other than case).
 *    But, this can be used to any client structure.
 *
 *    Returns:
 *      "name[user@ip#.port]" if 'showip' is true;
 *      "name[sockethost]", if name and sockhost are different and
 *      showip is false; else
 *      "name".
 *
 *  NOTE 1:
 *    Watch out the allocation of "nbuf", if either sptr->name
 *    or sptr->sockhost gets changed into pointers instead of
 *    directly allocated within the structure...
 *
 *  NOTE 2:
 *    Function return either a pointer to the structure (sptr) or
 *    to internal buffer (nbuf). *NEVER* use the returned pointer
 *    to modify what it points!!!
 */
char *get_client_name(aClient *sptr, int showip)
{
  static char nbuf[HOSTLEN * 2 + USERLEN + 5];

  if (MyConnect(sptr))
  {
    if (IsUnixSocket(sptr))
    {
      if (showip)
	sprintf_irc(nbuf, "%s[%s]", sptr->name, sptr->sockhost);
      else
	sprintf_irc(nbuf, "%s[%s]", sptr->name, me.sockhost);
    }
    else
    {
      if (showip)
	sprintf_irc(nbuf, "%s[%s@%s]", sptr->name,
	    (!(sptr->flags & FLAGS_GOTID)) ? "" :
	    sptr->username, inetntoa(sptr->ip));
      else
      {
	if (strCasediff(sptr->name, sptr->sockhost))
	  sprintf_irc(nbuf, "%s[%s]", sptr->name, sptr->sockhost);
	else
	  return sptr->name;
      }
    }
    return nbuf;
  }
  return sptr->name;
}

char *get_client_host(aClient *cptr)
{
  static char nbuf[HOSTLEN * 2 + USERLEN + 5];

  if (!MyConnect(cptr))
    return cptr->name;
  if (!cptr->hostp)
    return get_client_name(cptr, FALSE);
  if (IsUnixSocket(cptr))
    sprintf_irc(nbuf, "%s[%s]", cptr->name, me.name);
  else
    sprintf(nbuf, "%s[%-.*s@%-.*s]", cptr->name, USERLEN,
	(!(cptr->flags & FLAGS_GOTID)) ? "" : cptr->username,
	HOSTLEN, cptr->hostp->h_name);
  return nbuf;
}

/*
 * Form sockhost such that if the host is of form user@host, only the host
 * portion is copied.
 */
void get_sockhost(aClient *cptr, char *host)
{
  Reg3 char *s;
  if ((s = strchr(host, '@')))
    s++;
  else
    s = host;
  strncpy(cptr->sockhost, s, sizeof(cptr->sockhost) - 1);
}

/*
 * Return wildcard name of my server name according to given config entry
 * --Jto
 */
char *my_name_for_link(char *name, aConfItem *aconf)
{
  static char namebuf[HOSTLEN];
  register int count = aconf->port;
  register char *start = name;

  if (count <= 0 || count > 5)
    return start;

  while (count-- && name)
  {
    name++;
    name = strchr(name, '.');
  }
  if (!name)
    return start;

  namebuf[0] = '*';
  strncpy(&namebuf[1], name, HOSTLEN - 1);
  namebuf[HOSTLEN - 1] = '\0';

  return namebuf;
}

/*
 * exit_downlinks - added by Run 25-9-94
 *
 * Removes all clients and downlinks (+clients) of any server
 * QUITs are generated and sent to local users.
 *
 * cptr    : server that must have all dependents removed
 * sptr    : source who thought that this was a good idea
 * comment : comment sent as sign off message to local clients
 */
static void exit_downlinks(aClient *cptr, aClient *sptr, char *comment)
{
  Reg1 aClient *acptr;
  Reg2 Dlink *next;
  Reg3 Dlink *lp;
  aClient **acptrp;
  int i;

  /* Run over all its downlinks */
  for (lp = cptr->serv->down; lp; lp = next)
  {
    next = lp->next;
    acptr = lp->value.cptr;
    /* Remove the downlinks and client of the downlink */
    exit_downlinks(acptr, sptr, comment);
    /* Remove the downlink itself */
    exit_one_client(acptr, me.name);
  }
  /* Remove all clients of this server */
  acptrp = cptr->serv->client_list;
  for (i = 0; i <= cptr->serv->nn_mask; ++acptrp, ++i)
    if (*acptrp)
      exit_one_client(*acptrp, comment);
}

/*
 * exit_client, rewritten 25-9-94 by Run
 *
 * This function exits a client of *any* type (user, server, etc)
 * from this server. Also, this generates all necessary prototol
 * messages that this exit may cause.
 *
 * This function implicitly exits all other clients depending on
 * this connection.
 *
 * For convenience, this function returns a suitable value for
 * m_funtion return value:
 *
 *   CPTR_KILLED     if (cptr == bcptr)
 *   0                if (cptr != bcptr)
 *
 * This function can be called in two ways:
 * 1) From before or in parse(), exitting the 'cptr', in which case it was
 *    invoked as exit_client(cptr, cptr, &me,...), causing it to always
 *    return CPTR_KILLED.
 * 2) Via parse from a m_function call, in which case it was invoked as
 *    exit_client(cptr, acptr, sptr, ...). Here 'sptr' is known; the client
 *    that generated the message in a way that we can assume he already
 *    did remove acptr from memory himself (or in other cases we don't mind
 *    because he will be delinked.) Or invoked as:
 *    exit_client(cptr, acptr/sptr, &me, ...) when WE decide this one should
 *    be removed.
 * In general: No generated SQUIT or QUIT should be sent to source link
 * sptr->from. And CPTR_KILLED should be returned if cptr got removed (too).
 *
 * --Run
 */
int exit_client(
    aClient *cptr,		    /* Connection being handled by
				       read_message right now */
    aClient *bcptr,		    /* Client being killed */
    aClient *sptr,		    /* The client that made the decision
				       to remove this one, never NULL */
    char *comment)		/* Reason for the exit */
{
  Reg1 aClient *acptr;
  Reg3 Dlink *dlp;
#ifdef	FNAME_USERLOG
  time_t on_for;
#endif
  char comment1[HOSTLEN + HOSTLEN + 2];

  if (MyConnect(bcptr))
  {
    bcptr->flags |= FLAGS_CLOSING;

/* New code - StyX */

#ifdef ALLOW_SNO_CONNEXIT
    if (IsUser(bcptr))
    {
	sendto_ops("Client Exiting: %s (%s@%s)", bcptr->name, bcptr->user->username, bcptr->user->host);
	sendbufto_op_mask(SNO_CONNEXIT);
    }
#endif

    update_load();
#ifdef FNAME_USERLOG
    on_for = now - bcptr->firsttime;
#if defined(USE_SYSLOG) && defined(SYSLOG_USERS)
    if (IsUser(bcptr))
      syslog(LOG_NOTICE, "%s (%3d:%02d:%02d): %s@%s (%s)\n",
	  myctime(bcptr->firsttime), on_for / 3600, (on_for % 3600) / 60,
	  on_for % 60, bcptr->user->username, bcptr->sockhost, bcptr->name);
#else
    {
      char linebuf[160];
      int logfile;

      /*
       * This conditional makes the logfile active only after
       * it's been created - thus logging can be turned off by
       * removing the file.
       *
       * stop NFS hangs...most systems should be able to open a
       * file in 3 seconds. -avalon (curtesy of wumpus)
       */
      alarm(3);
      if (IsUser(bcptr) &&
	  (logfile = open(FNAME_USERLOG, O_WRONLY | O_APPEND)) != -1)
      {
	alarm(0);
	sprintf(linebuf,
	    "%s (%3d:%02d:%02d): %s@%s [%s]\n",
	    myctime(bcptr->firsttime),
	    on_for / 3600, (on_for % 3600) / 60,
	    on_for % 60,
	    bcptr->user->username, bcptr->user->host,
	    bcptr->username);
	alarm(3);
	write(logfile, linebuf, strlen(linebuf));
	alarm(0);
	close(logfile);
      }
      alarm(0);
      /* Modification by stealth@caen.engin.umich.edu */
    }
#endif
#endif
    if (bcptr != sptr->from	    /* The source knows already */
	&& IsClient(bcptr))	/* Not a Ping struct or Log file */
    {
      if (IsServer(bcptr) || IsHandshake(bcptr))
	sendto_one(bcptr, ":%s SQUIT %s 0 :%s", sptr->name,
	    me.name, comment);
      else if (!IsConnecting(bcptr))
	sendto_one(bcptr, "ERROR :Lien coup: %s by %s (%s)",
	    get_client_name(bcptr, FALSE), sptr->name, comment);
      if ((IsServer(bcptr) || IsHandshake(bcptr) || IsConnecting(bcptr)) &&
	  (sptr == &me || (IsServer(sptr) &&
	  (strncmp(comment, "Leaf-only link", 14) ||
	  strncmp(comment, "Ce Link n'est pas celui d'un HUB !", 12)))))
      {
	if (bcptr->serv->user && *bcptr->serv->by &&
	    (acptr = findNUser(bcptr->serv->by)) &&
	    acptr->user == bcptr->serv->user)
	{
	  if (MyUser(acptr) || Protocol(acptr->from) < 10)
	    sendto_one(acptr,
		":%s NOTICE %s :Lien avec %s annul: %s",
		me.name, acptr->name, bcptr->name, comment);
	  else
	    sendto_one(acptr,
		"%c NOTICE %c%c%c :Lien avec %s annul: %s",
		NumServ(&me), NumNick(acptr), bcptr->name, comment);
	}
	else
	  acptr = NULL;
	if (sptr == &me)
	  sendto_lops_butone(acptr, "Lien avec %s annul: %s",
	      bcptr->name, comment);
      }
    }
    /*
     *  Close the Client connection first.
     */
    close_connection(bcptr);
  }

  if (IsServer(bcptr))
  {
    strcpy(comment1, bcptr->serv->up->name);
    strcat(comment1, " ");
    strcat(comment1, bcptr->name);
    if (IsUser(sptr))
      sendto_lops_butone(sptr, "%s SQUIT by %s [%s]:",
	  (sptr->user->server == bcptr ||
	  sptr->user->server == bcptr->serv->up) ? "Local" : "Remote",
	  get_client_name(sptr, TRUE), sptr->user->server->name);
    else if (sptr != &me && bcptr->serv->up != sptr)
      sendto_ops("Recu SQUIT %s depuis %s :", bcptr->name,
	  IsServer(sptr) ? sptr->name : get_client_name(sptr, TRUE));
    sendto_op_mask(SNO_NETWORK, "NetSplit: %s (%s)", comment1, comment);
  }

  /*
   * First generate the needed protocol for the other server links
   * except the source:
   */
  for (dlp = me.serv->down; dlp; dlp = dlp->next)
    if (dlp->value.cptr != sptr->from && dlp->value.cptr != bcptr)
    {
      if (IsServer(bcptr))
	sendto_one(dlp->value.cptr, ":%s SQUIT %s %lu :%s",
	    sptr->name, bcptr->name, bcptr->serv->timestamp, comment);
      else if (IsUser(bcptr) && (bcptr->flags & FLAGS_KILLED) == 0)
	sendto_one(dlp->value.cptr, ":%s QUIT :%s", bcptr->name, comment);
    }

  /* Then remove the client structures */
  if (IsServer(bcptr))
    exit_downlinks(bcptr, sptr, comment1);
  exit_one_client(bcptr, comment);

  /*
   *  cptr can only have been killed if it was cptr itself that got killed here,
   *  because cptr can never have been a dependant of bcptr    --Run
   */
  return (cptr == bcptr) ? CPTR_KILLED : 0;
}

/*
 * Exit client with formatted message, added 25-9-94 by Run
 */
int vexit_client_msg(aClient *cptr, aClient *bcptr, aClient *sptr,
    char *pattern, va_list vl)
{
  char msgbuf[1024];
  vsprintf_irc(msgbuf, pattern, vl);
  return exit_client(cptr, bcptr, sptr, msgbuf);
}

int exit_client_msg(aClient *cptr, aClient *bcptr,
    aClient *sptr, char *pattern, ...)
{
  va_list vl;
  char msgbuf[1024];

  va_start(vl, pattern);
  vsprintf_irc(msgbuf, pattern, vl);
  va_end(vl);

  return exit_client(cptr, bcptr, sptr, msgbuf);
}

/*
 * Exit one client, local or remote. Assuming for local client that
 * all dependants already have been removed, and socket is closed.
 *
 * Rewritten by Run - 24 sept 94
 *
 * bcptr : client being (s)quitted
 * sptr : The source (prefix) of the QUIT or SQUIT
 *
 * --Run
 */
static void exit_one_client(aClient *bcptr, char *comment)
{
  Link *lp;

  if (bcptr->serv && bcptr->serv->client_list)	/* Was SetServerYXX called ? */
    ClearServerYXX(bcptr);	/* Removes server from server_list[] */
  if (IsUser(bcptr))
  {
    /* Stop a running /LIST clean */
    if (MyUser(bcptr) && bcptr->listing)
    {
      bcptr->listing->chptr->mode.mode &= ~MODE_LISTED;
      RunFree(bcptr->listing);
      bcptr->listing = NULL;
    }

    if (AskedPing(bcptr))
      cancel_ping(bcptr, NULL);
    /*
     * If a person is on a channel, send a QUIT notice
     * to every client (person) on the same channel (so
     * that the client can show the "**signoff" message).
     * (Note: The notice is to the local clients *only*)
     */
    sendto_common_channels(bcptr, ":%s QUIT :%s", bcptr->name, comment);

    while ((lp = bcptr->user->channel))
      remove_user_from_channel(bcptr, lp->value.chptr);

    /* Clean up invitefield */
    while ((lp = bcptr->user->invited))
      del_invite(bcptr, lp->value.chptr);

    /* Clean up silencefield */
    while ((lp = bcptr->user->silence))
      del_silence(bcptr, lp->value.cp);

    if (IsInvisible(bcptr))
      --nrof.inv_clients;
    if (IsOper(bcptr))
      --nrof.opers;
    if (MyConnect(bcptr))
      Count_clientdisconnects(bcptr, nrof);
    else
      Count_remoteclientquits(nrof);
  }
  else if (IsServer(bcptr))
  {
    /* Remove downlink list node of uplink */
    remove_dlink(&bcptr->serv->up->serv->down, bcptr->serv->updown);

    if (MyConnect(bcptr))
      Count_serverdisconnects(nrof);
    else
      Count_remoteserverquits(nrof);
  }
  else if (IsPing(bcptr))	/* Apperently, we are closing ALL links */
  {
    del_queries((char *)bcptr);
    end_ping(bcptr);
    return;
  }
  else if (IsMe(bcptr))
  {
    sendto_ops("ERROR: il essai de me tej ! : %s", comment);
    return;			/* ...must *never* exit self! */
  }
  else if (IsUnknown(bcptr) || IsConnecting(bcptr) || IsHandshake(bcptr))
    Count_unknowndisconnects(nrof);

  /* Update IPregistry */
  if (IsIPChecked(bcptr))
    IPcheck_disconnect(bcptr);

  /* Remove from serv->client_list */
  if (bcptr->user && *bcptr->yxx)
    bcptr->user->server->serv->client_list[Index(bcptr, bcptr->user->server)] = NULL;

  /* Remove bcptr from the client list */
#ifdef DEBUGMODE
  if (hRemClient(bcptr) != 0)
    Debug((DEBUG_ERROR, "%p !in tab %s[%s] %p %p %p %d %d %p",
	bcptr, bcptr->name, bcptr->from ? bcptr->from->sockhost : "??host",
	bcptr->from, bcptr->next, bcptr->prev, bcptr->fd,
	bcptr->status, bcptr->user));
#else
  hRemClient(bcptr);
#endif
  remove_client_from_list(bcptr);
  return;
}

void checklist(void)
{
  Reg1 aClient *acptr;
  Reg2 int i, j;

  if (!(bootopt & BOOT_AUTODIE))
    return;
  for (j = i = 0; i <= highest_fd; i++)
    if (!(acptr = loc_clients[i]))
      continue;
    else if (IsUser(acptr))
      j++;
  if (!j)
  {
#ifdef	USE_SYSLOG
    syslog(LOG_WARNING, "ircd exiting: autodie");
#endif
    exit(0);
  }
  return;
}

void initstats(void)
{
  memset(&ircst, 0, sizeof(ircst));
}

void tstats(aClient *cptr, char *name)
{
  Reg1 aClient *acptr;
  Reg2 int i;
  Reg3 struct stats *sp;
  struct stats tmp;

  sp = &tmp;
  memcpy(sp, ircstp, sizeof(*sp));
  for (i = 0; i < MAXCONNECTIONS; i++)
  {
    if (!(acptr = loc_clients[i]))
      continue;
    if (IsServer(acptr))
    {
      sp->is_sbs += acptr->sendB;
      sp->is_sbr += acptr->receiveB;
      sp->is_sks += acptr->sendK;
      sp->is_skr += acptr->receiveK;
      sp->is_sti += now - acptr->firsttime;
      sp->is_sv++;
      if (sp->is_sbs > 1023)
      {
	sp->is_sks += (sp->is_sbs >> 10);
	sp->is_sbs &= 0x3ff;
      }
      if (sp->is_sbr > 1023)
      {
	sp->is_skr += (sp->is_sbr >> 10);
	sp->is_sbr &= 0x3ff;
      }
    }
    else if (IsUser(acptr))
    {
      sp->is_cbs += acptr->sendB;
      sp->is_cbr += acptr->receiveB;
      sp->is_cks += acptr->sendK;
      sp->is_ckr += acptr->receiveK;
      sp->is_cti += now - acptr->firsttime;
      sp->is_cl++;
      if (sp->is_cbs > 1023)
      {
	sp->is_cks += (sp->is_cbs >> 10);
	sp->is_cbs &= 0x3ff;
      }
      if (sp->is_cbr > 1023)
      {
	sp->is_ckr += (sp->is_cbr >> 10);
	sp->is_cbr &= 0x3ff;
      }
    }
    else if (IsUnknown(acptr))
      sp->is_ni++;
  }

  sendto_one(cptr, ":%s %d %s :accepts %u refused %u",
      me.name, RPL_STATSDEBUG, name, sp->is_ac, sp->is_ref);
  sendto_one(cptr, ":%s %d %s :unknown commands %u prefixes %u",
      me.name, RPL_STATSDEBUG, name, sp->is_unco, sp->is_unpf);
  sendto_one(cptr, ":%s %d %s :nick collisions %u unknown closes %u",
      me.name, RPL_STATSDEBUG, name, sp->is_kill, sp->is_ni);
  sendto_one(cptr, ":%s %d %s :wrong direction %u empty %u",
      me.name, RPL_STATSDEBUG, name, sp->is_wrdi, sp->is_empt);
  sendto_one(cptr, ":%s %d %s :numerics seen %u mode fakes %u",
      me.name, RPL_STATSDEBUG, name, sp->is_num, sp->is_fake);
  sendto_one(cptr, ":%s %d %s :auth successes %u fails %u",
      me.name, RPL_STATSDEBUG, name, sp->is_asuc, sp->is_abad);
  sendto_one(cptr, ":%s %d %s :local connections %u udp packets %u",
      me.name, RPL_STATSDEBUG, name, sp->is_loc, sp->is_udp);
  sendto_one(cptr, ":%s %d %s :Client Server",
      me.name, RPL_STATSDEBUG, name);
  sendto_one(cptr, ":%s %d %s :connected %u %u",
      me.name, RPL_STATSDEBUG, name, sp->is_cl, sp->is_sv);
  sendto_one(cptr, ":%s %d %s :bytes sent %lu.%uK %lu.%uK",
      me.name, RPL_STATSDEBUG, name,
      sp->is_cks, sp->is_cbs, sp->is_sks, sp->is_sbs);
  sendto_one(cptr, ":%s %d %s :bytes recv %lu.%uK %lu.%uK",
      me.name, RPL_STATSDEBUG, name,
      sp->is_ckr, sp->is_cbr, sp->is_skr, sp->is_sbr);
  sendto_one(cptr, ":%s %d %s :time connected %lu %lu",
      me.name, RPL_STATSDEBUG, name, sp->is_cti, sp->is_sti);
}

